# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'observatories.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormObservatory(object):
    def setupUi(self, FormObservatory):
        FormObservatory.setObjectName("FormObservatory")
        FormObservatory.resize(400, 215)
        self.gridLayout_2 = QtWidgets.QGridLayout(FormObservatory)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.comboBoxObservatory = QtWidgets.QComboBox(FormObservatory)
        self.comboBoxObservatory.setObjectName("comboBoxObservatory")
        self.comboBoxObservatory.addItem("")
        self.gridLayout_2.addWidget(self.comboBoxObservatory, 0, 0, 1, 3)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.lineEditName = QtWidgets.QLineEdit(FormObservatory)
        self.lineEditName.setObjectName("lineEditName")
        self.gridLayout.addWidget(self.lineEditName, 0, 0, 1, 2)
        self.label = QtWidgets.QLabel(FormObservatory)
        self.label.setMinimumSize(QtCore.QSize(91, 18))
        self.label.setMaximumSize(QtCore.QSize(91, 18))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.doubleSpinBoxLatitude = QtWidgets.QDoubleSpinBox(FormObservatory)
        self.doubleSpinBoxLatitude.setMinimum(-90.0)
        self.doubleSpinBoxLatitude.setMaximum(90.0)
        self.doubleSpinBoxLatitude.setObjectName("doubleSpinBoxLatitude")
        self.gridLayout.addWidget(self.doubleSpinBoxLatitude, 1, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(FormObservatory)
        self.label_2.setMinimumSize(QtCore.QSize(91, 18))
        self.label_2.setMaximumSize(QtCore.QSize(91, 18))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.doubleSpinBoxLongitude = QtWidgets.QDoubleSpinBox(FormObservatory)
        self.doubleSpinBoxLongitude.setMinimum(-180.0)
        self.doubleSpinBoxLongitude.setMaximum(180.0)
        self.doubleSpinBoxLongitude.setProperty("value", 0.0)
        self.doubleSpinBoxLongitude.setObjectName("doubleSpinBoxLongitude")
        self.gridLayout.addWidget(self.doubleSpinBoxLongitude, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(FormObservatory)
        self.label_3.setMinimumSize(QtCore.QSize(91, 18))
        self.label_3.setMaximumSize(QtCore.QSize(91, 18))
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.doubleSpinBoxAltitude = QtWidgets.QDoubleSpinBox(FormObservatory)
        self.doubleSpinBoxAltitude.setMinimum(0.0)
        self.doubleSpinBoxAltitude.setMaximum(1e+56)
        self.doubleSpinBoxAltitude.setProperty("value", 0.0)
        self.doubleSpinBoxAltitude.setObjectName("doubleSpinBoxAltitude")
        self.gridLayout.addWidget(self.doubleSpinBoxAltitude, 3, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 1, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 2, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(291, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 3, 0, 1, 1)
        self.pushButtonRremove = QtWidgets.QPushButton(FormObservatory)
        self.pushButtonRremove.setEnabled(False)
        self.pushButtonRremove.setObjectName("pushButtonRremove")
        self.gridLayout_2.addWidget(self.pushButtonRremove, 3, 1, 1, 1)
        self.pushButtonSave = QtWidgets.QPushButton(FormObservatory)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.gridLayout_2.addWidget(self.pushButtonSave, 3, 2, 1, 1)

        self.retranslateUi(FormObservatory)
        QtCore.QMetaObject.connectSlotsByName(FormObservatory)

    def retranslateUi(self, FormObservatory):
        _translate = QtCore.QCoreApplication.translate
        FormObservatory.setWindowTitle(_translate("FormObservatory", "Observatories"))
        self.comboBoxObservatory.setItemText(0, _translate("FormObservatory", "New"))
        self.lineEditName.setPlaceholderText(_translate("FormObservatory", "Name"))
        self.label.setText(_translate("FormObservatory", "Latitude"))
        self.label_2.setText(_translate("FormObservatory", "Longitude"))
        self.label_3.setText(_translate("FormObservatory", "Altitude"))
        self.pushButtonRremove.setText(_translate("FormObservatory", "Remove"))
        self.pushButtonSave.setText(_translate("FormObservatory", "Add/Update"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormObservatory = QtWidgets.QWidget()
    ui = Ui_FormObservatory()
    ui.setupUi(FormObservatory)
    FormObservatory.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'align.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormAlign(object):
    def setupUi(self, FormAlign):
        FormAlign.setObjectName("FormAlign")
        FormAlign.resize(419, 197)
        self.gridLayout_2 = QtWidgets.QGridLayout(FormAlign)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_4 = QtWidgets.QLabel(FormAlign)
        self.label_4.setMinimumSize(QtCore.QSize(154, 26))
        self.label_4.setMaximumSize(QtCore.QSize(154, 26))
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 0, 0, 1, 1)
        self.comboBoxReference = QtWidgets.QComboBox(FormAlign)
        self.comboBoxReference.setObjectName("comboBoxReference")
        self.gridLayout.addWidget(self.comboBoxReference, 0, 1, 1, 1)
        self.pushButtonShow = QtWidgets.QPushButton(FormAlign)
        self.pushButtonShow.setMinimumSize(QtCore.QSize(80, 26))
        self.pushButtonShow.setMaximumSize(QtCore.QSize(80, 26))
        self.pushButtonShow.setObjectName("pushButtonShow")
        self.gridLayout.addWidget(self.pushButtonShow, 0, 2, 1, 1)
        self.label = QtWidgets.QLabel(FormAlign)
        self.label.setMinimumSize(QtCore.QSize(154, 27))
        self.label.setMaximumSize(QtCore.QSize(154, 27))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.spinBoxMaxControlPoint = QtWidgets.QSpinBox(FormAlign)
        self.spinBoxMaxControlPoint.setProperty("value", 50)
        self.spinBoxMaxControlPoint.setObjectName("spinBoxMaxControlPoint")
        self.gridLayout.addWidget(self.spinBoxMaxControlPoint, 1, 1, 1, 2)
        self.label_2 = QtWidgets.QLabel(FormAlign)
        self.label_2.setMinimumSize(QtCore.QSize(154, 27))
        self.label_2.setMaximumSize(QtCore.QSize(154, 27))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.doubleSpinDetectionSigma = QtWidgets.QDoubleSpinBox(FormAlign)
        self.doubleSpinDetectionSigma.setMinimum(-1e+42)
        self.doubleSpinDetectionSigma.setMaximum(1e+42)
        self.doubleSpinDetectionSigma.setProperty("value", 5.0)
        self.doubleSpinDetectionSigma.setObjectName("doubleSpinDetectionSigma")
        self.gridLayout.addWidget(self.doubleSpinDetectionSigma, 2, 1, 1, 2)
        self.label_3 = QtWidgets.QLabel(FormAlign)
        self.label_3.setMinimumSize(QtCore.QSize(154, 27))
        self.label_3.setMaximumSize(QtCore.QSize(154, 27))
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.spinBoxMinArea = QtWidgets.QSpinBox(FormAlign)
        self.spinBoxMinArea.setProperty("value", 50)
        self.spinBoxMinArea.setObjectName("spinBoxMinArea")
        self.gridLayout.addWidget(self.spinBoxMinArea, 3, 1, 1, 2)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 42, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 1, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(211, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 2, 0, 1, 1)
        self.pushButtonGO = QtWidgets.QPushButton(FormAlign)
        self.pushButtonGO.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setObjectName("pushButtonGO")
        self.gridLayout_2.addWidget(self.pushButtonGO, 2, 1, 1, 1)

        self.retranslateUi(FormAlign)
        QtCore.QMetaObject.connectSlotsByName(FormAlign)

    def retranslateUi(self, FormAlign):
        _translate = QtCore.QCoreApplication.translate
        FormAlign.setWindowTitle(_translate("FormAlign", "Align"))
        self.label_4.setText(_translate("FormAlign", "Reference"))
        self.pushButtonShow.setText(_translate("FormAlign", "Show"))
        self.label.setText(_translate("FormAlign", "Max Control Points"))
        self.label_2.setText(_translate("FormAlign", "Detection Sigma"))
        self.label_3.setText(_translate("FormAlign", "Min Area"))
        self.pushButtonGO.setText(_translate("FormAlign", ":go"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormAlign = QtWidgets.QWidget()
    ui = Ui_FormAlign()
    ui.setupUi(FormAlign)
    FormAlign.show()
    sys.exit(app.exec_())



from .about import Ui_FormAbout
from .align import Ui_FormAlign
from .arithmetic import Ui_FormArithmetic
from .bin import Ui_FormBin
from .ccdproc import Ui_FormCCDPROC
from .combine import Ui_FormCombine
from .cosmic_cleaner import Ui_FormCosmicCleaner
from .crop import Ui_FormCrop
from .display import Ui_FormDisplay
from .header import Ui_FormHeader
from .header_calculator import Ui_FormHeaderCalculator
from .hedit import Ui_FormHedit
from .hselect import Ui_FormHSelect
from .log import Ui_FormLog
from .main import Ui_MainWindow
from .observatories import Ui_FormObservatory
from .photometry import Ui_FormPhotometry
from .rotate import Ui_FormRotate
from .settings import Ui_FormSettings
from .shift import Ui_FormShift
from .statics import Ui_FormStatics
from .wcs import Ui_FormWCS



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Optional, List, Union, Any, TYPE_CHECKING, Dict, Callable

import numpy as np
from astropy.time import Time
from typing_extensions import Self

if TYPE_CHECKING:
    from .fits import Fits

from astropy.io.fits import Header
from sep import Background

import pandas as pd
from astropy.nddata import CCDData
from astropy.coordinates import SkyCoord

NUMERICS = Union[float, int, List[Union[float, int]]]


class Data(ABC):
    high_precision = False

    @classmethod
    @abstractmethod
    def from_image(cls, path: str) -> Self:
        ...

    @classmethod
    @abstractmethod
    def from_path(cls, path: str) -> Self:
        ...

    @classmethod
    @abstractmethod
    def from_data_header(cls, data: Any, header: Optional[Header] = None,
                         output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @classmethod
    @abstractmethod
    def sample(cls) -> Self:
        ...

    @abstractmethod
    def reset_zmag(self) -> None:
        ...

    @abstractmethod
    def header(self) -> pd.DataFrame:
        ...

    @abstractmethod
    def data(self) -> Any:
        ...

    @abstractmethod
    def value(self, x: int, y: int) -> float:
        ...

    @abstractmethod
    def pure_header(self) -> Header:
        ...

    @abstractmethod
    def ccd(self) -> CCDData:
        ...

    @abstractmethod
    def imstat(self) -> pd.DataFrame:
        ...

    @abstractmethod
    def cosmic_clean(self, output: Optional[str] = None,
                     override: bool = False, sigclip: float = 4.5,
                     sigfrac: float = 0.3, objlim: int = 5, gain: float = 1.0,
                     readnoise: float = 6.5, satlevel: float = 65535.0,
                     niter: int = 4, sepmed: bool = True,
                     cleantype: str = 'meanmask', fsmode: str = 'median',
                     psfmodel: str = 'gauss', psffwhm: float = 2.5,
                     psfsize: int = 7, psfk: Optional[Any] = None,
                     psfbeta: float = 4.765, gain_apply: bool = True) -> Self:
        ...

    @abstractmethod
    def hedit(self, keys: Union[str, List[str]],
              values: Optional[Union[str, int, float, bool, List[Union[str, int, float, bool]]]] = None,
              comments: Optional[Union[str, List[str]]] = None, delete: bool = False, value_is_key: bool = False
              ) -> Self:
        ...

    @abstractmethod
    def save_as(self, output: str, override: bool = False) -> Self:
        ...

    @abstractmethod
    def add(self, other: Union[Self, float, int],
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def sub(self, other: Union[Self, int, float],
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def mul(self, other: Union[Self, int, float],
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def div(self, other: Union[Self, int, float],
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def pow(self, other: Union[Fits, float, int],
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def imarith(self, other: Union[Self, int, float], operand: str,
                output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def align(self, reference: Self, output: Optional[str] = None,
              max_control_points: int = 50, min_area: int = 5,
              override: bool = False) -> Self:
        ...

    @abstractmethod
    def show(self, scale: bool = True, sources: Optional[pd.DataFrame] = None) -> None:
        ...

    @abstractmethod
    def coordinate_picker(self, scale: bool = True) -> pd.DataFrame:
        ...

    @abstractmethod
    def solve_field(self, api_key: str, solve_timeout: int = 120,
                    force_image_upload: bool = False,
                    output: Optional[str] = None, override: bool = False
                    ) -> Self:
        ...

    @abstractmethod
    def zero_correction(self, master_zero: Self,
                        output: Optional[str] = None, override: bool = True, force: bool = False) -> Self:
        ...

    @abstractmethod
    def dark_correction(self, master_dark: Self, exposure: Optional[str] = None,
                        output: Optional[str] = None, override: bool = False,
                        force: bool = False) -> Self:
        ...

    @abstractmethod
    def flat_correction(self, master_flat: Self, output: Optional[str] = None,
                        override: bool = False, force: bool = False) -> Self:
        ...

    @abstractmethod
    def ccdproc(self, master_zero: Optional[Self] = None, master_dark: Optional[Self] = None,
                master_flat: Optional[Self] = None, exposure: Optional[str] = None, output: Optional[str] = None,
                override: bool = False, force: bool = False) -> Self:
        ...

    @abstractmethod
    def background(self) -> Background:
        ...

    @abstractmethod
    def daofind(self, sigma: float = 3.0, fwhm: float = 3.0,
                threshold: float = 5.0) -> pd.DataFrame:
        ...

    @abstractmethod
    def extract(self, detection_sigma: float = 5.0,
                min_area: float = 5.0) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry_sep(self,
                       xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                       headers: Optional[Union[str, list[str]]] = None,
                       exposure: Optional[Union[str, float, int]] = None
                       ) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry_phu(self, xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                       headers: Optional[Union[str, list[str]]] = None,
                       exposure: Optional[Union[str, float, int]] = None
                       ) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry(self, xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                   headers: Optional[Union[str, list[str]]] = None,
                   exposure: Optional[Union[str, float, int]] = None
                   ) -> pd.DataFrame:
        ...

    @abstractmethod
    def shift(self, x: int, y: int, output: Optional[str] = None,
              override: bool = False) -> Self:
        ...

    @abstractmethod
    def rotate(self, angle: Union[float, int], output: Optional[str] = None,
               override: bool = False) -> Self:
        ...

    @abstractmethod
    def crop(self, x: int, y: int, width: int, height: int,
             output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def bin(self, binning_factor: Union[int, List[int]], func: Callable[[Any], float] = np.mean,
            output: Optional[str] = None, override: bool = False) -> Self:
        ...

    @abstractmethod
    def pixels_to_skys(self, xs: Union[List[Union[int, float]], int, float],
                       ys: Union[List[Union[int, float]], int, float]) -> pd.DataFrame:
        ...

    @abstractmethod
    def skys_to_pixels(self, skys: Union[List[SkyCoord], SkyCoord]) -> pd.DataFrame:
        ...

    @abstractmethod
    def map_to_sky(self):
        ...


class DataArray(ABC):
    high_precision = False

    @classmethod
    @abstractmethod
    def from_video(cls, path: str, start_time: Optional[Union[Time, float]] = None) -> Self:
        ...

    @classmethod
    @abstractmethod
    def from_paths(cls, paths: List[str]) -> Self:
        ...

    @classmethod
    @abstractmethod
    def from_pattern(cls, pattern: str) -> Self:
        ...

    @classmethod
    @abstractmethod
    def sample(cls, numer_of_samples: int = 10) -> Self:
        ...

    @abstractmethod
    def merge(self, other: Self) -> None:
        ...

    @abstractmethod
    def append(self, other: Fits) -> None:
        ...

    @abstractmethod
    def header(self) -> pd.DataFrame:
        ...

    @abstractmethod
    def data(self) -> List[Any]:
        ...

    @abstractmethod
    def value(self, x: int, y: int) -> pd.DataFrame:
        ...

    @abstractmethod
    def pure_header(self) -> List[Header]:
        ...

    @abstractmethod
    def ccd(self) -> List[CCDData]:
        ...

    @abstractmethod
    def imstat(self) -> pd.DataFrame:
        ...

    @abstractmethod
    def cosmic_clean(self, output: Optional[str] = None,
                     override: bool = False, sigclip: float = 4.5,
                     sigfrac: float = 0.3, objlim: int = 5, gain: float = 1.0,
                     readnoise: float = 6.5, satlevel: float = 65535.0,
                     niter: int = 4, sepmed: bool = True,
                     cleantype: str = 'meanmask', fsmode: str = 'median',
                     psfmodel: str = 'gauss', psffwhm: float = 2.5,
                     psfsize: int = 7, psfk: Optional[Any] = None,
                     psfbeta: float = 4.765, gain_apply: bool = True) -> Self:
        ...

    @abstractmethod
    def hedit(self, keys: Union[str, List[str]],
              values: Optional[Union[str, int, float, bool, List[Union[str, int, float, bool]]]] = None,
              comments: Optional[Union[str, List[str]]] = None, delete: bool = False, value_is_key: bool = False
              ) -> Self:
        ...

    @abstractmethod
    def hselect(self, fields: Union[str, List[str]]) -> pd.DataFrame:
        ...

    @abstractmethod
    def save_as(self, output: str) -> Self:
        ...

    @abstractmethod
    def add(self, other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]],
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def sub(self, other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]],
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def mul(self, other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]],
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def div(self, other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]],
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def pow(self,
            other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]],
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def imarith(self, other: Union[Self, Fits, float, int, List[Union[Fits, float, int]]], operand: str,
                output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def shift(self, xs: Union[List[int], int], ys: Union[List[int], int],
              output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def rotate(self, angle: Union[List[Union[float, int]], float, int],
               output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def crop(self, xs: Union[List[int], int], ys: Union[List[int], int],
             widths: Union[List[int], int], heights: Union[List[int], int],
             output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def bin(self, binning_factor: Union[int, List[int]], func: Callable[[Any], float] = np.mean,
            output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def align(self, other: Union[Fits, int] = 0, output: Optional[str] = None,
              max_control_points: int = 50, min_area: int = 5) -> Self:
        ...

    @abstractmethod
    def show(self, scale: bool = True, interval: float = 1.0) -> None:
        ...

    @abstractmethod
    def solve_field(self, api_key: str, reference: Union[Fits, int] = 0,
                    solve_timeout: int = 120, force_image_upload: bool = False,
                    max_control_points: int = 50, min_area: int = 5,
                    output: Optional[str] = None) -> Self:
        ...

    @abstractmethod
    def zero_correction(self, master_zero: Fits,
                        output: Optional[str] = None, force: bool = False) -> Self:
        ...

    @abstractmethod
    def dark_correction(self, master_dark: Fits, exposure: Optional[str] = None,
                        output: Optional[str] = None, force: bool = False) -> Self:
        ...

    @abstractmethod
    def flat_correction(self, master_flat: Fits,
                        output: Optional[str] = None, force: bool = False) -> Self:
        ...

    @abstractmethod
    def ccdproc(self, master_zero: Optional[Fits] = None, master_dark: Optional[Fits] = None,
                master_flat: Optional[Fits] = None, exposure: Optional[str] = None, output: Optional[str] = None,
                force: bool = False) -> Self:
        ...

    @abstractmethod
    def background(self) -> List[Background]:
        ...

    @abstractmethod
    def daofind(self, index: int = 0, sigma: float = 3.0, fwhm: float = 3.0,
                threshold: float = 5.0) -> pd.DataFrame:
        ...

    @abstractmethod
    def extract(self, index: int = 0, detection_sigma: float = 5.0,
                min_area: float = 5.0) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry_sep(self, xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                       headers: Optional[Union[str, list[str]]] = None,
                       exposure: Optional[Union[str, float, int]] = None
                       ) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry_phu(self, xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                       headers: Optional[Union[str, list[str]]] = None,
                       exposure: Optional[Union[str, float, int]] = None
                       ) -> pd.DataFrame:
        ...

    @abstractmethod
    def photometry(self, xs: NUMERICS, ys: NUMERICS, rs: NUMERICS,
                   headers: Optional[Union[str, list[str]]] = None,
                   exposure: Optional[Union[str, float, int]] = None
                   ) -> pd.DataFrame:
        ...

    @abstractmethod
    def group_by(self, groups: Union[str, List[str]]) -> Dict[Any, Self]:
        ...

    @abstractmethod
    def combine(self, method: str = "average", clipping: Optional[str] = None,
                weights: Optional[List[Union[float, int]]] = None,
                output: Optional[str] = None, override: bool = False) -> Fits:
        ...

    @abstractmethod
    def zero_combine(self, method: str = "median", clipping: Optional[str] = None,
                     output: Optional[str] = None, override: bool = False) -> Fits:
        ...

    @abstractmethod
    def dark_combine(self, method: str = "median", clipping: Optional[str] = None,
                     weights: Optional[Union[List[str], List[Union[float, int]]]] = None,
                     output: Optional[str] = None, override: bool = False) -> Fits:
        ...

    @abstractmethod
    def flat_combine(self, method: str = "median", clipping: Optional[str] = None,
                     weights: Optional[Union[List[str], List[Union[float, int]]]] = None,
                     output: Optional[str] = None, override: bool = False) -> Fits:
        ...

    @abstractmethod
    def pixels_to_skys(self, xs: Union[List[Union[int, float]], int, float],
                       ys: Union[List[Union[int, float]], int, float]) -> pd.DataFrame:
        ...

    @abstractmethod
    def skys_to_pixels(self, skys: Union[List[SkyCoord], SkyCoord]) -> pd.DataFrame:
        ...

    @abstractmethod
    def map_to_sky(self) -> pd.DataFrame:
        ...



from logging import getLogger, Logger
from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QMessageBox, QTreeWidgetItem, QDialog, QVBoxLayout, QListWidget, QDialogButtonBox

from myraflib import Fits

SCHEMA = {"primary": "#F5AE71"}

from PyQt5 import QtCore, QtWidgets, QtGui
import sys


class QToaster(QtWidgets.QFrame):
    closed = QtCore.pyqtSignal()

    def __init__(self, *args, **kwargs):
        super(QToaster, self).__init__(*args, **kwargs)
        QtWidgets.QHBoxLayout(self)

        self.setSizePolicy(QtWidgets.QSizePolicy.Maximum,
                           QtWidgets.QSizePolicy.Maximum)

        self.setStyleSheet('''
            QToaster {
                border: 1px solid black;
                border-radius: 0px; 
                color: rgb(255, 255, 255);
                background-color: rgb(57, 66, 81);
            }
        ''')

        self.timer = QtCore.QTimer(singleShot=True, timeout=self.hide)

        if self.parent():
            self.opacityEffect = QtWidgets.QGraphicsOpacityEffect(opacity=0)
            self.setGraphicsEffect(self.opacityEffect)
            self.opacityAni = QtCore.QPropertyAnimation(self.opacityEffect, b'opacity')
            self.parent().installEventFilter(self)
        else:
            self.opacityAni = QtCore.QPropertyAnimation(self, b'windowOpacity')
        self.opacityAni.setStartValue(0.)
        self.opacityAni.setEndValue(1.)
        self.opacityAni.setDuration(100)
        self.opacityAni.finished.connect(self.checkClosed)

        self.corner = QtCore.Qt.TopLeftCorner
        self.margin = 10

    def checkClosed(self):
        if self.opacityAni.direction() == self.opacityAni.Backward:
            self.close()

    def restore(self):
        self.timer.stop()
        self.opacityAni.stop()
        if self.parent():
            self.opacityEffect.setOpacity(1)
        else:
            self.setWindowOpacity(1)

    def hide(self):
        self.opacityAni.setDirection(self.opacityAni.Backward)
        self.opacityAni.setDuration(500)
        self.opacityAni.start()

    def eventFilter(self, source, event):
        if source == self.parent() and event.type() == QtCore.QEvent.Resize:
            self.opacityAni.stop()
            parentRect = self.parent().rect()
            geo = self.geometry()
            if self.corner == QtCore.Qt.TopLeftCorner:
                geo.moveTopLeft(
                    parentRect.topLeft() + QtCore.QPoint(self.margin, self.margin))
            elif self.corner == QtCore.Qt.TopRightCorner:
                geo.moveTopRight(
                    parentRect.topRight() + QtCore.QPoint(-self.margin, self.margin))
            elif self.corner == QtCore.Qt.BottomRightCorner:
                geo.moveBottomRight(
                    parentRect.bottomRight() + QtCore.QPoint(-self.margin, -self.margin))
            else:
                geo.moveBottomLeft(
                    parentRect.bottomLeft() + QtCore.QPoint(self.margin, -self.margin))
            self.setGeometry(geo)
            self.restore()
            self.timer.start()
        return super(QToaster, self).eventFilter(source, event)

    def enterEvent(self, event):
        self.restore()

    def leaveEvent(self, event):
        self.timer.start()

    def closeEvent(self, event):
        self.deleteLater()

    def resizeEvent(self, event):
        super(QToaster, self).resizeEvent(event)
        if not self.parent():
            path = QtGui.QPainterPath()
            path.addRoundedRect(QtCore.QRectF(self.rect()).translated(-.5, -.5), 4, 4)
            self.setMask(QtGui.QRegion(path.toFillPolygon(QtGui.QTransform()).toPolygon()))
        else:
            self.clearMask()

    @staticmethod
    def show_message(parent, message,
                     icon=QtWidgets.QStyle.SP_MessageBoxInformation,
                     corner=QtCore.Qt.TopLeftCorner, margin=10, closable=True,
                     timeout=5000, desktop=False, parentWindow=True):

        if parent and parentWindow:
            parent = parent.window()

        if not parent or desktop:
            self = QToaster(None)
            self.setWindowFlags(self.windowFlags() | QtCore.Qt.FramelessWindowHint |
                                QtCore.Qt.BypassWindowManagerHint)
            self.__self = self

            currentScreen = QtWidgets.QApplication.primaryScreen()
            if parent and parent.window().geometry().size().isValid():
                reference = parent.window().geometry()
            else:
                reference = QtCore.QRect(
                    QtGui.QCursor.pos() - QtCore.QPoint(1, 1),
                    QtCore.QSize(3, 3))
            maxArea = 0
            for screen in QtWidgets.QApplication.screens():
                intersected = screen.geometry().intersected(reference)
                area = intersected.width() * intersected.height()
                if area > maxArea:
                    maxArea = area
                    currentScreen = screen
            parentRect = currentScreen.availableGeometry()
        else:
            self = QToaster(parent)
            parentRect = parent.rect()

        self.timer.setInterval(timeout)

        self.label = QtWidgets.QLabel(message)
        self.label.setStyleSheet("color: rgb(255, 255, 255);")
        font = QtGui.QFont()
        # font.setFamily("IRANYekanWeb")
        font.setPointSize(12)
        font.setWeight(100)
        self.label.setFont(font)
        self.layout().addWidget(self.label)

        if closable:
            self.closeButton = QtWidgets.QToolButton()
            self.layout().addWidget(self.closeButton)
            closeIcon = self.style().standardIcon(
                QtWidgets.QStyle.SP_TitleBarCloseButton)
            self.closeButton.setIcon(closeIcon)
            self.closeButton.setAutoRaise(True)
            self.closeButton.clicked.connect(self.close)

        self.timer.start()

        self.raise_()
        self.adjustSize()

        self.corner = corner
        self.margin = margin

        geo = self.geometry()
        if corner == QtCore.Qt.TopLeftCorner:
            geo.moveTopLeft(
                parentRect.topLeft() + QtCore.QPoint(margin, margin))
        elif corner == QtCore.Qt.TopRightCorner:
            geo.moveTopRight(
                parentRect.topRight() + QtCore.QPoint(-margin, margin))
        elif corner == QtCore.Qt.BottomRightCorner:
            geo.moveBottomRight(
                parentRect.bottomRight() + QtCore.QPoint(-margin, -margin))
        else:
            geo.moveBottomLeft(
                parentRect.bottomLeft() + QtCore.QPoint(margin, -margin))

        self.setGeometry(geo)
        self.show()
        self.opacityAni.start()


class MultiSelectDialog(QDialog):
    def __init__(self, items, parent=None):
        super().__init__(parent)
        self.selected_items = []

        self.setWindowTitle('Select Items')

        layout = QVBoxLayout(self)

        self.list_widget = QListWidget(self)
        self.list_widget.addItems(items)
        self.list_widget.setSelectionMode(QListWidget.MultiSelection)
        layout.addWidget(self.list_widget)

        self.button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, self)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)
        layout.addWidget(self.button_box)

    def accept(self):
        self.selected_items = [item.text() for item in self.list_widget.selectedItems()]
        super().accept()


class CustomQTreeWidgetItem(QTreeWidgetItem):
    def __hash__(self):
        return id(self)


class GUIFunctions:
    def __init__(self, parent: QtWidgets.QMainWindow, logger: Logger=None):

        if logger is None:
            self.logger = getLogger(__file__)
        else:
            self.logger = logger

        self.parent = parent

    def error(self, text):
        QMessageBox.critical(self.parent, "MYRaf", text)

    def warning(self, text):
        QMessageBox.warning(self.parent, "MYRaf", text)

    def get_file(self, caption, file_type=None):
        try:
            if file_type is not None:
                file_type_to_use = file_type
            else:
                file_type_to_use = "fits, fit, fts (*.fits *.fit *.fts)"

            file, _ = QtWidgets.QFileDialog.getOpenFileName(self.parent, caption, '', file_type_to_use)
            return file
        except Exception as e:
            self.logger.error(e)
            self.parent.gui_functions.toast(self.parent, str(e))
            return ""

    def get_files(self, caption, file_type=None):
        try:
            if file_type is not None:
                file_type_to_use = file_type
            else:
                file_type_to_use = "fits, fit, fts (*.fits *.fit *.fts)"

            files, _ = QtWidgets.QFileDialog.getOpenFileNames(self.parent, caption, '', file_type_to_use)
            return files
        except Exception as e:
            self.logger.error(e)
            self.parent.gui_functions.toast(self.parent, str(e))
            return []

    def get_directory(self,  caption):
        try:
            directory = QtWidgets.QFileDialog.getExistingDirectory(self.parent, caption)
            return directory
        except Exception as e:
            self.logger.error(e)
            self.parent.gui_functions.toast(self.parent, str(e))
            return ""

    def save_file(self,  caption, file_type_to_use):
        file, _ = QtWidgets.QFileDialog.getSaveFileName(self.parent, caption, filter=file_type_to_use)

        return file

    def add_to_files(self,  images_to_add, device, grp="Group"):
        if len(images_to_add) == 0:
            return

        try:
            progress = QtWidgets.QProgressDialog("Adding files...", "Abort", 0, len(images_to_add), self.parent)

            progress.setWindowModality(QtCore.Qt.WindowModal)
            progress.setFixedSize(progress.sizeHint() + QSize(400, 0))
            progress.setWindowTitle('MYRaf: Please Wait')
            progress.setAutoClose(True)

            group_layer = CustomQTreeWidgetItem(device, [grp])
            for iteration, file in enumerate(images_to_add):
                fits = Fits.from_path(file)
                progress.setLabelText(f"Adding {fits.file.name}")

                if progress.wasCanceled():
                    progress.setLabelText("ABORT!")
                    break

                group_layer.setFirstColumnSpanned(True)

                file_name_layer = CustomQTreeWidgetItem(group_layer, [fits.file.name])
                file_name_layer.setFirstColumnSpanned(True)

                item = CustomQTreeWidgetItem(file_name_layer, ["Path", fits.file.resolve().parent.__str__()])
                item.setFlags(QtCore.Qt.ItemIsEnabled)
                statistics = fits.imstat()
                for key, value in statistics.iloc[0].items():
                    item = CustomQTreeWidgetItem(file_name_layer, [key.capitalize(), f"{value:.2f}"])
                    item.setFlags(QtCore.Qt.ItemIsEnabled)

                progress.setValue(iteration)

            progress.close()

        except Exception as e:
            self.logger.error(e)
            self.parent.gui_functions.toast(self.parent, str(e))

    def get_selected_files(self, tree_widget):
        selected_items_dict = {}

        def traverse_item(item: QTreeWidgetItem):
            if item.isSelected():
                selected_items_dict[item] = []
                child_count = item.childCount()
                for i in range(child_count):
                    child = item.child(i)
                    selected_items_dict[item].append(child)
            else:
                child_selected = []
                child_count = item.childCount()
                for i in range(child_count):
                    child = item.child(i)
                    if child.isSelected():
                        child_selected.append(child)
                if child_selected:
                    selected_items_dict[item] = child_selected

        root_count = tree_widget.topLevelItemCount()
        for i in range(root_count):
            root = tree_widget.topLevelItem(i)
            traverse_item(root)

        return selected_items_dict

    def remove_from_files(self, tree_widget):
        selected_items_dict = self.get_selected_files(tree_widget)
        for parent, children in selected_items_dict.items():
            for child in children:
                parent.removeChild(child)

            if parent.childCount() == 0:
                tree_widget.takeTopLevelItem(tree_widget.indexOfTopLevelItem(parent))

    def get_number(self, caption, text, min_val=0, max_val=100, step_val=1, default=0):
        number, ok = QtWidgets.QInputDialog.getInt(
            self.parent, caption, text, min=min_val, max=max_val, step=step_val, value=default
        )
        return number, ok

    def get_text(self,  caption: str, text: str, default: str = None):
        text, ok = QtWidgets.QInputDialog.getText(self.parent, caption, text, text=default)
        return text, ok

    def get_item(self,  caption, label, items):
        item, ok = QtWidgets.QInputDialog.getItem(self.parent, caption, label, items)
        return item, ok

    def ask(self,  caption, question):
        answer = QtWidgets.QMessageBox.question(self.parent, caption, question,
                                                QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                QtWidgets.QMessageBox.No)

        return answer == QtWidgets.QMessageBox.Yes

    def add_to_combo(self, combo_widget, items):
        combo_widget.addItems(items)

    def clear_table(self, table_widget):
        while table_widget.rowCount() > 0:
            table_widget.removeRow(0)

    def get_from_table(self, table_widget):
        number_of_rows = table_widget.rowCount()
        number_of_columns = table_widget.columnCount()
        ret = []
        if number_of_rows > 0 and number_of_columns > 0:
            for i in range(number_of_rows):
                row = []
                for j in range(number_of_columns):
                    row.append(table_widget.item(i, j).text())

                ret.append(row)

            return ret

        return []

    def get_from_table_selected(self, table_widget):
        ret = []
        for i in table_widget.selectionModel().selectedRows():
            row = []
            for j in range(table_widget.columnCount()):
                row.append(table_widget.item(i.row(), j).text())
            ret.append(row)

        return ret

    def add_to_table(self, table_widget, data):
        for line in data:
            row_position = table_widget.rowCount()
            table_widget.insertRow(row_position)
            for it, value in enumerate(line):
                table_widget.setItem(row_position, it, QtWidgets.QTableWidgetItem(str(value)))

    def remove_from_table(self, table_widget):
        selected_rows = list(set([index.row() for index in table_widget.selectedIndexes()]))
        selected_rows.sort(reverse=True)
        for row in selected_rows:
            table_widget.removeRow(row)

    def add_to_list(self, list_widget, data):
        it = list_widget.count() - 1
        for x in data:
            it = it + 1
            item = QtWidgets.QListWidgetItem()
            list_widget.addItem(item)
            item = list_widget.item(it)
            item.setText(x)

    def get_from_list(self, list_widget):
        return [
            list_widget.item(x).text()
            for x in range(list_widget.count())
        ]

    def get_from_list_selected(self, list_widget):
        return [
            x.text()
            for x in list_widget.selectedItems()
        ]

    def remove_from_list(self, list_widget):
        for x in list_widget.selectedItems():
            list_widget.takeItem(list_widget.row(x))

    def toast(self, message,
              icon=QtWidgets.QStyle.SP_MessageBoxInformation,
              corner=QtCore.Qt.BottomRightCorner, margin=10, closable=True,
              timeout=5000, desktop=False, parentWindow=True):

        if self.parent and parentWindow:
            parent = self.parent.window()

        if not parent or desktop:
            toaster = QToaster(None)
            toaster.setWindowFlags(toaster.windowFlags() | QtCore.Qt.FramelessWindowHint |
                                   QtCore.Qt.BypassWindowManagerHint)

            toaster.__self = toaster

            currentScreen = QtWidgets.QApplication.primaryScreen()
            if parent and parent.window().geometry().size().isValid():
                reference = parent.window().geometry()
            else:
                reference = QtCore.QRect(
                    QtGui.QCursor.pos() - QtCore.QPoint(1, 1),
                    QtCore.QSize(3, 3))
            maxArea = 0
            for screen in QtWidgets.QApplication.screens():
                intersected = screen.geometry().intersected(reference)
                area = intersected.width() * intersected.height()
                if area > maxArea:
                    maxArea = area
                    currentScreen = screen
            parentRect = currentScreen.availableGeometry()
        else:
            toaster = QToaster(parent)
            parentRect = parent.rect()

        toaster.timer.setInterval(timeout)

        toaster.label = QtWidgets.QLabel(message)
        toaster.label.setStyleSheet("color: rgb(255, 255, 255);")
        font = QtGui.QFont()
        # font.setFamily("IRANYekanWeb")
        font.setPointSize(12)
        font.setWeight(100)
        toaster.label.setFont(font)
        toaster.layout().addWidget(toaster.label)

        if closable:
            toaster.closeButton = QtWidgets.QToolButton()
            toaster.layout().addWidget(toaster.closeButton)
            closeIcon = toaster.style().standardIcon(
                QtWidgets.QStyle.SP_TitleBarCloseButton)
            toaster.closeButton.setIcon(closeIcon)
            toaster.closeButton.setAutoRaise(True)
            toaster.closeButton.clicked.connect(toaster.close)

        toaster.timer.start()

        toaster.raise_()
        toaster.adjustSize()

        toaster.corner = corner
        toaster.margin = margin

        geo = toaster.geometry()
        if corner == QtCore.Qt.TopLeftCorner:
            geo.moveTopLeft(
                parentRect.topLeft() + QtCore.QPoint(margin, margin))
        elif corner == QtCore.Qt.TopRightCorner:
            geo.moveTopRight(
                parentRect.topRight() + QtCore.QPoint(-margin, margin))
        elif corner == QtCore.Qt.BottomRightCorner:
            geo.moveBottomRight(
                parentRect.bottomRight() + QtCore.QPoint(-margin, -margin))
        else:
            geo.moveBottomLeft(
                parentRect.bottomLeft() + QtCore.QPoint(margin, -margin))

        toaster.setGeometry(geo)
        toaster.show()
        toaster.opacityAni.start()

    def collapse_all_children(self, tree_widget):
        top_level_items = self.get_selected_files(tree_widget)
        for top_level_item in top_level_items.keys():
            for i in range(top_level_item.childCount()):
                child_item = top_level_item.child(i)
                tree_widget.collapseItem(child_item)

    def expand_all_children(self, tree_widget):
        top_level_items = self.get_selected_files(tree_widget)
        for top_level_item in top_level_items.keys():
            for i in range(top_level_item.childCount()):
                child_item = top_level_item.child(i)
                tree_widget.collapseItem(child_item)
                tree_widget.expandItem(child_item)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rotate.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormRotate(object):
    def setupUi(self, FormRotate):
        FormRotate.setObjectName("FormRotate")
        FormRotate.resize(792, 643)
        self.gridLayout_3 = QtWidgets.QGridLayout(FormRotate)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.groupBox = QtWidgets.QGroupBox(FormRotate)
        self.groupBox.setMinimumSize(QtCore.QSize(437, 592))
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3.addWidget(self.groupBox, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(FormRotate)
        self.groupBox_2.setMinimumSize(QtCore.QSize(331, 0))
        self.groupBox_2.setMaximumSize(QtCore.QSize(331, 16777215))
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.tableWidgetAmount = QtWidgets.QTableWidget(self.groupBox_2)
        self.tableWidgetAmount.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidgetAmount.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableWidgetAmount.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidgetAmount.setObjectName("tableWidgetAmount")
        self.tableWidgetAmount.setColumnCount(2)
        self.tableWidgetAmount.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetAmount.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetAmount.setHorizontalHeaderItem(1, item)
        self.gridLayout.addWidget(self.tableWidgetAmount, 0, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_2, 0, 1, 1, 1)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem = QtWidgets.QSpacerItem(519, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 0, 0, 1, 1)
        self.pushButtonGo = QtWidgets.QPushButton(FormRotate)
        self.pushButtonGo.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGo.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGo.setObjectName("pushButtonGo")
        self.gridLayout_2.addWidget(self.pushButtonGo, 0, 1, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout_2, 1, 0, 1, 2)

        self.retranslateUi(FormRotate)
        QtCore.QMetaObject.connectSlotsByName(FormRotate)

    def retranslateUi(self, FormRotate):
        _translate = QtCore.QCoreApplication.translate
        FormRotate.setWindowTitle(_translate("FormRotate", "Rotate"))
        self.groupBox.setTitle(_translate("FormRotate", "Image"))
        self.groupBox_2.setTitle(_translate("FormRotate", "Amount"))
        item = self.tableWidgetAmount.horizontalHeaderItem(0)
        item.setText(_translate("FormRotate", "Image"))
        item = self.tableWidgetAmount.horizontalHeaderItem(1)
        item.setText(_translate("FormRotate", "Angle"))
        self.pushButtonGo.setText(_translate("FormRotate", ":go"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormRotate = QtWidgets.QWidget()
    ui = Ui_FormRotate()
    ui.setupUi(FormRotate)
    FormRotate.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1245, 1002)
        MainWindow.setMinimumSize(QtCore.QSize(800, 600))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.playGround = QtWidgets.QMdiArea(self.centralwidget)
        self.playGround.setObjectName("playGround")
        self.gridLayout_2.addWidget(self.playGround, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget_2 = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_2.setFloating(False)
        self.dockWidget_2.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget_2.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dockWidget_2.setObjectName("dockWidget_2")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.gridLayout = QtWidgets.QGridLayout(self.dockWidgetContents_2)
        self.gridLayout.setObjectName("gridLayout")
        self.treeWidget = QtWidgets.QTreeWidget(self.dockWidgetContents_2)
        self.treeWidget.setEditTriggers(QtWidgets.QAbstractItemView.DoubleClicked|QtWidgets.QAbstractItemView.EditKeyPressed)
        self.treeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.treeWidget.setObjectName("treeWidget")
        self.gridLayout.addWidget(self.treeWidget, 0, 1, 1, 1)
        self.dockWidget_2.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget_2)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1245, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuSettings = QtWidgets.QMenu(self.menubar)
        self.menuSettings.setObjectName("menuSettings")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.actionDefaults = QtWidgets.QAction(MainWindow)
        self.actionDefaults.setObjectName("actionDefaults")
        self.actionAdd = QtWidgets.QAction(MainWindow)
        self.actionAdd.setObjectName("actionAdd")
        self.actionSubtract = QtWidgets.QAction(MainWindow)
        self.actionSubtract.setObjectName("actionSubtract")
        self.actionMultiply = QtWidgets.QAction(MainWindow)
        self.actionMultiply.setObjectName("actionMultiply")
        self.actionDivide = QtWidgets.QAction(MainWindow)
        self.actionDivide.setObjectName("actionDivide")
        self.actionPower = QtWidgets.QAction(MainWindow)
        self.actionPower.setObjectName("actionPower")
        self.actionMod = QtWidgets.QAction(MainWindow)
        self.actionMod.setObjectName("actionMod")
        self.actionStatics = QtWidgets.QAction(MainWindow)
        self.actionStatics.setObjectName("actionStatics")
        self.actionHedit = QtWidgets.QAction(MainWindow)
        self.actionHedit.setObjectName("actionHedit")
        self.actionCosmic_Cleaner = QtWidgets.QAction(MainWindow)
        self.actionCosmic_Cleaner.setObjectName("actionCosmic_Cleaner")
        self.actionAbsolute = QtWidgets.QAction(MainWindow)
        self.actionAbsolute.setObjectName("actionAbsolute")
        self.actionBackground = QtWidgets.QAction(MainWindow)
        self.actionBackground.setObjectName("actionBackground")
        self.actionsep = QtWidgets.QAction(MainWindow)
        self.actionsep.setObjectName("actionsep")
        self.actiondaofind = QtWidgets.QAction(MainWindow)
        self.actiondaofind.setObjectName("actiondaofind")
        self.actionAlign = QtWidgets.QAction(MainWindow)
        self.actionAlign.setObjectName("actionAlign")
        self.actionSolve = QtWidgets.QAction(MainWindow)
        self.actionSolve.setObjectName("actionSolve")
        self.actionShift = QtWidgets.QAction(MainWindow)
        self.actionShift.setObjectName("actionShift")
        self.actionRotate = QtWidgets.QAction(MainWindow)
        self.actionRotate.setObjectName("actionRotate")
        self.actionCrop = QtWidgets.QAction(MainWindow)
        self.actionCrop.setObjectName("actionCrop")
        self.actionBin = QtWidgets.QAction(MainWindow)
        self.actionBin.setObjectName("actionBin")
        self.actionShow = QtWidgets.QAction(MainWindow)
        self.actionShow.setObjectName("actionShow")
        self.actionCombine = QtWidgets.QAction(MainWindow)
        self.actionCombine.setObjectName("actionCombine")
        self.actionCalibration_2 = QtWidgets.QAction(MainWindow)
        self.actionCalibration_2.setObjectName("actionCalibration_2")
        self.actionCombine_3 = QtWidgets.QAction(MainWindow)
        self.actionCombine_3.setObjectName("actionCombine_3")
        self.actionZero_Combine = QtWidgets.QAction(MainWindow)
        self.actionZero_Combine.setObjectName("actionZero_Combine")
        self.actionDark_Combine = QtWidgets.QAction(MainWindow)
        self.actionDark_Combine.setObjectName("actionDark_Combine")
        self.actionFlat_Combine = QtWidgets.QAction(MainWindow)
        self.actionFlat_Combine.setObjectName("actionFlat_Combine")
        self.actionZero_Correction = QtWidgets.QAction(MainWindow)
        self.actionZero_Correction.setObjectName("actionZero_Correction")
        self.actionDark_Correction = QtWidgets.QAction(MainWindow)
        self.actionDark_Correction.setObjectName("actionDark_Correction")
        self.actionFlat_Correction = QtWidgets.QAction(MainWindow)
        self.actionFlat_Correction.setObjectName("actionFlat_Correction")
        self.actionPixel_to_World = QtWidgets.QAction(MainWindow)
        self.actionPixel_to_World.setObjectName("actionPixel_to_World")
        self.actionWorld_to_Pixel = QtWidgets.QAction(MainWindow)
        self.actionWorld_to_Pixel.setObjectName("actionWorld_to_Pixel")
        self.actionsep_2 = QtWidgets.QAction(MainWindow)
        self.actionsep_2.setObjectName("actionsep_2")
        self.actionphotutils = QtWidgets.QAction(MainWindow)
        self.actionphotutils.setObjectName("actionphotutils")
        self.actionAll = QtWidgets.QAction(MainWindow)
        self.actionAll.setObjectName("actionAll")
        self.actionAdd_2 = QtWidgets.QAction(MainWindow)
        self.actionAdd_2.setObjectName("actionAdd_2")
        self.actionRemove = QtWidgets.QAction(MainWindow)
        self.actionRemove.setObjectName("actionRemove")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionHeader_Calculator = QtWidgets.QAction(MainWindow)
        self.actionHeader_Calculator.setObjectName("actionHeader_Calculator")
        self.actionObservatories = QtWidgets.QAction(MainWindow)
        self.actionObservatories.setObjectName("actionObservatories")
        self.actionLog = QtWidgets.QAction(MainWindow)
        self.actionLog.setObjectName("actionLog")
        self.actionccdproc = QtWidgets.QAction(MainWindow)
        self.actionccdproc.setObjectName("actionccdproc")
        self.actionPhotometry = QtWidgets.QAction(MainWindow)
        self.actionPhotometry.setObjectName("actionPhotometry")
        self.actionSettings = QtWidgets.QAction(MainWindow)
        self.actionSettings.setObjectName("actionSettings")
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuSettings.addAction(self.actionSettings)
        self.menuSettings.addAction(self.actionObservatories)
        self.menuHelp.addAction(self.actionLog)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuSettings.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MYRaf V3"))
        self.treeWidget.headerItem().setText(0, _translate("MainWindow", "key"))
        self.treeWidget.headerItem().setText(1, _translate("MainWindow", "Value"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuSettings.setTitle(_translate("MainWindow", "Settings"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionDefaults.setText(_translate("MainWindow", "Defaults..."))
        self.actionAdd.setText(_translate("MainWindow", "Add..."))
        self.actionSubtract.setText(_translate("MainWindow", "Subtract..."))
        self.actionMultiply.setText(_translate("MainWindow", "Multiply..."))
        self.actionDivide.setText(_translate("MainWindow", "Divide..."))
        self.actionPower.setText(_translate("MainWindow", "Power..."))
        self.actionMod.setText(_translate("MainWindow", "Mod..."))
        self.actionStatics.setText(_translate("MainWindow", "Statics..."))
        self.actionHedit.setText(_translate("MainWindow", "Hedit..."))
        self.actionCosmic_Cleaner.setText(_translate("MainWindow", "Cosmic Cleaner..."))
        self.actionAbsolute.setText(_translate("MainWindow", "Absolute..."))
        self.actionBackground.setText(_translate("MainWindow", "Background..."))
        self.actionsep.setText(_translate("MainWindow", "sep..."))
        self.actiondaofind.setText(_translate("MainWindow", "daofind..."))
        self.actionAlign.setText(_translate("MainWindow", "Align..."))
        self.actionSolve.setText(_translate("MainWindow", "Solve..."))
        self.actionShift.setText(_translate("MainWindow", "Shift..."))
        self.actionRotate.setText(_translate("MainWindow", "Rotate..."))
        self.actionCrop.setText(_translate("MainWindow", "Crop..."))
        self.actionBin.setText(_translate("MainWindow", "Bin..."))
        self.actionShow.setText(_translate("MainWindow", "Show..."))
        self.actionCombine.setText(_translate("MainWindow", "Combine"))
        self.actionCalibration_2.setText(_translate("MainWindow", "Calibration"))
        self.actionCombine_3.setText(_translate("MainWindow", "Combine..."))
        self.actionZero_Combine.setText(_translate("MainWindow", "Zero Combine..."))
        self.actionDark_Combine.setText(_translate("MainWindow", "Dark Combine..."))
        self.actionFlat_Combine.setText(_translate("MainWindow", "Flat Combine..."))
        self.actionZero_Correction.setText(_translate("MainWindow", "Zero Correction..."))
        self.actionDark_Correction.setText(_translate("MainWindow", "Dark Correction..."))
        self.actionFlat_Correction.setText(_translate("MainWindow", "Flat Correction..."))
        self.actionPixel_to_World.setText(_translate("MainWindow", "Pixel to World..."))
        self.actionWorld_to_Pixel.setText(_translate("MainWindow", "World to Pixel..."))
        self.actionsep_2.setText(_translate("MainWindow", "sep..."))
        self.actionphotutils.setText(_translate("MainWindow", "photutils..."))
        self.actionAll.setText(_translate("MainWindow", "All..."))
        self.actionAdd_2.setText(_translate("MainWindow", "Add"))
        self.actionRemove.setText(_translate("MainWindow", "Remove"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionHeader_Calculator.setText(_translate("MainWindow", "Header Calculator..."))
        self.actionObservatories.setText(_translate("MainWindow", "Observatories"))
        self.actionLog.setText(_translate("MainWindow", "Log"))
        self.actionccdproc.setText(_translate("MainWindow", "ccdproc"))
        self.actionPhotometry.setText(_translate("MainWindow", "Photometry"))
        self.actionSettings.setText(_translate("MainWindow", "Settings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bin.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormBin(object):
    def setupUi(self, FormBin):
        FormBin.setObjectName("FormBin")
        FormBin.resize(288, 127)
        self.gridLayout_3 = QtWidgets.QGridLayout(FormBin)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButtonGo = QtWidgets.QPushButton(FormBin)
        self.pushButtonGo.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGo.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGo.setObjectName("pushButtonGo")
        self.gridLayout_3.addWidget(self.pushButtonGo, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(519, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 2, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 4, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem1, 1, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(FormBin)
        self.groupBox_2.setMinimumSize(QtCore.QSize(0, 72))
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.groupBox_2)
        self.label.setMinimumSize(QtCore.QSize(0, 18))
        self.label.setMaximumSize(QtCore.QSize(16, 18))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.spinBoxXAmount = QtWidgets.QSpinBox(self.groupBox_2)
        self.spinBoxXAmount.setMinimumSize(QtCore.QSize(100, 25))
        self.spinBoxXAmount.setMaximumSize(QtCore.QSize(16777215, 25))
        self.spinBoxXAmount.setMinimum(1)
        self.spinBoxXAmount.setMaximum(65535)
        self.spinBoxXAmount.setObjectName("spinBoxXAmount")
        self.gridLayout.addWidget(self.spinBoxXAmount, 0, 1, 1, 1)
        self.line = QtWidgets.QFrame(self.groupBox_2)
        self.line.setMinimumSize(QtCore.QSize(0, 27))
        self.line.setMaximumSize(QtCore.QSize(16, 27))
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 0, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox_2)
        self.label_2.setMinimumSize(QtCore.QSize(0, 18))
        self.label_2.setMaximumSize(QtCore.QSize(16, 18))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 3, 1, 1)
        self.spinBoxYAmount = QtWidgets.QSpinBox(self.groupBox_2)
        self.spinBoxYAmount.setEnabled(True)
        self.spinBoxYAmount.setMinimumSize(QtCore.QSize(100, 25))
        self.spinBoxYAmount.setMaximumSize(QtCore.QSize(16777215, 25))
        self.spinBoxYAmount.setMinimum(1)
        self.spinBoxYAmount.setMaximum(65535)
        self.spinBoxYAmount.setObjectName("spinBoxYAmount")
        self.gridLayout.addWidget(self.spinBoxYAmount, 0, 4, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_2, 0, 0, 1, 2)

        self.retranslateUi(FormBin)
        QtCore.QMetaObject.connectSlotsByName(FormBin)

    def retranslateUi(self, FormBin):
        _translate = QtCore.QCoreApplication.translate
        FormBin.setWindowTitle(_translate("FormBin", "Bin"))
        self.pushButtonGo.setText(_translate("FormBin", ":go"))
        self.groupBox_2.setTitle(_translate("FormBin", "Amount"))
        self.label.setText(_translate("FormBin", "X"))
        self.label_2.setText(_translate("FormBin", "Y"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormBin = QtWidgets.QWidget()
    ui = Ui_FormBin()
    ui.setupUi(FormBin)
    FormBin.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'photometry.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormPhotometry(object):
    def setupUi(self, FormPhotometry):
        FormPhotometry.setObjectName("FormPhotometry")
        FormPhotometry.resize(834, 807)
        self.gridLayout_4 = QtWidgets.QGridLayout(FormPhotometry)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.pushButtonGO = QtWidgets.QPushButton(FormPhotometry)
        self.pushButtonGO.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setObjectName("pushButtonGO")
        self.gridLayout_4.addWidget(self.pushButtonGO, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(848, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 1, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(FormPhotometry)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_2.setMinimumSize(QtCore.QSize(241, 0))
        self.groupBox_2.setMaximumSize(QtCore.QSize(241, 16777215))
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.tableWidgetCoordinates = QtWidgets.QTableWidget(self.groupBox_2)
        self.tableWidgetCoordinates.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidgetCoordinates.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidgetCoordinates.setObjectName("tableWidgetCoordinates")
        self.tableWidgetCoordinates.setColumnCount(2)
        self.tableWidgetCoordinates.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetCoordinates.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetCoordinates.setHorizontalHeaderItem(1, item)
        self.gridLayout.addWidget(self.tableWidgetCoordinates, 0, 0, 1, 1)
        self.gridLayout_6.addWidget(self.groupBox_2, 0, 0, 2, 1)
        self.labelRa = QtWidgets.QLabel(self.tab)
        self.labelRa.setMinimumSize(QtCore.QSize(0, 19))
        self.labelRa.setMaximumSize(QtCore.QSize(16777215, 19))
        self.labelRa.setText("")
        self.labelRa.setObjectName("labelRa")
        self.gridLayout_6.addWidget(self.labelRa, 0, 1, 1, 1)
        self.labelDec = QtWidgets.QLabel(self.tab)
        self.labelDec.setMinimumSize(QtCore.QSize(0, 19))
        self.labelDec.setMaximumSize(QtCore.QSize(16777215, 19))
        self.labelDec.setText("")
        self.labelDec.setObjectName("labelDec")
        self.gridLayout_6.addWidget(self.labelDec, 0, 2, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_6.addWidget(self.groupBox, 1, 1, 1, 2)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.label = QtWidgets.QLabel(self.tab_2)
        self.label.setObjectName("label")
        self.gridLayout_10.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.tab_2)
        self.label_2.setObjectName("label_2")
        self.gridLayout_10.addWidget(self.label_2, 2, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.doubleSpinBoxApertureRadius = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.doubleSpinBoxApertureRadius.setMinimum(1.0)
        self.doubleSpinBoxApertureRadius.setMaximum(9999999.0)
        self.doubleSpinBoxApertureRadius.setObjectName("doubleSpinBoxApertureRadius")
        self.gridLayout_3.addWidget(self.doubleSpinBoxApertureRadius, 0, 0, 1, 1)
        self.pushButtonAddToRadii = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonAddToRadii.setMaximumSize(QtCore.QSize(80, 26))
        self.pushButtonAddToRadii.setObjectName("pushButtonAddToRadii")
        self.gridLayout_3.addWidget(self.pushButtonAddToRadii, 0, 1, 1, 1)
        self.listWidgetApertureRadii = QtWidgets.QListWidget(self.groupBox_3)
        self.listWidgetApertureRadii.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidgetApertureRadii.setObjectName("listWidgetApertureRadii")
        self.gridLayout_3.addWidget(self.listWidgetApertureRadii, 1, 0, 1, 2)
        self.gridLayout_10.addWidget(self.groupBox_3, 4, 0, 1, 2)
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.listWidgetHeaders = QtWidgets.QListWidget(self.groupBox_4)
        self.listWidgetHeaders.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidgetHeaders.setObjectName("listWidgetHeaders")
        self.gridLayout_2.addWidget(self.listWidgetHeaders, 0, 0, 4, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 56, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 0, 1, 1, 1)
        self.listWidgetHeadersToExtract = QtWidgets.QListWidget(self.groupBox_4)
        self.listWidgetHeadersToExtract.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidgetHeadersToExtract.setObjectName("listWidgetHeadersToExtract")
        self.gridLayout_2.addWidget(self.listWidgetHeadersToExtract, 0, 2, 4, 1)
        self.pushButtonAddToHeaderToExtract = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButtonAddToHeaderToExtract.setObjectName("pushButtonAddToHeaderToExtract")
        self.gridLayout_2.addWidget(self.pushButtonAddToHeaderToExtract, 1, 1, 1, 1)
        self.pushButtonRemoveFromHeaderToExtract = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButtonRemoveFromHeaderToExtract.setObjectName("pushButtonRemoveFromHeaderToExtract")
        self.gridLayout_2.addWidget(self.pushButtonRemoveFromHeaderToExtract, 2, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 57, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 3, 1, 1, 1)
        self.gridLayout_10.addWidget(self.groupBox_4, 5, 0, 1, 2)
        self.tabWidget_2 = QtWidgets.QTabWidget(self.tab_2)
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.gridLayout_5 = QtWidgets.QGridLayout()
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_3 = QtWidgets.QLabel(self.tab_3)
        self.label_3.setMinimumSize(QtCore.QSize(100, 0))
        self.label_3.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_3.setObjectName("label_3")
        self.gridLayout_5.addWidget(self.label_3, 0, 0, 1, 1)
        self.doubleSpinBoxSexSigma = QtWidgets.QDoubleSpinBox(self.tab_3)
        self.doubleSpinBoxSexSigma.setMinimum(1.0)
        self.doubleSpinBoxSexSigma.setProperty("value", 5.0)
        self.doubleSpinBoxSexSigma.setObjectName("doubleSpinBoxSexSigma")
        self.gridLayout_5.addWidget(self.doubleSpinBoxSexSigma, 0, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tab_3)
        self.label_4.setMinimumSize(QtCore.QSize(100, 0))
        self.label_4.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_4.setObjectName("label_4")
        self.gridLayout_5.addWidget(self.label_4, 1, 0, 1, 1)
        self.doubleSpinBoxSexMinimumArea = QtWidgets.QDoubleSpinBox(self.tab_3)
        self.doubleSpinBoxSexMinimumArea.setMinimum(1.0)
        self.doubleSpinBoxSexMinimumArea.setProperty("value", 5.0)
        self.doubleSpinBoxSexMinimumArea.setObjectName("doubleSpinBoxSexMinimumArea")
        self.gridLayout_5.addWidget(self.doubleSpinBoxSexMinimumArea, 1, 1, 1, 1)
        self.gridLayout_7.addLayout(self.gridLayout_5, 0, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 33, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem3, 1, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_3, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.tab_4)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.gridLayout_8 = QtWidgets.QGridLayout()
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_5 = QtWidgets.QLabel(self.tab_4)
        self.label_5.setMinimumSize(QtCore.QSize(100, 0))
        self.label_5.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_5.setObjectName("label_5")
        self.gridLayout_8.addWidget(self.label_5, 0, 0, 1, 1)
        self.doubleSpinBoxDAOFindSigma = QtWidgets.QDoubleSpinBox(self.tab_4)
        self.doubleSpinBoxDAOFindSigma.setMinimum(1.0)
        self.doubleSpinBoxDAOFindSigma.setProperty("value", 3.0)
        self.doubleSpinBoxDAOFindSigma.setObjectName("doubleSpinBoxDAOFindSigma")
        self.gridLayout_8.addWidget(self.doubleSpinBoxDAOFindSigma, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.tab_4)
        self.label_6.setMinimumSize(QtCore.QSize(100, 0))
        self.label_6.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_6.setObjectName("label_6")
        self.gridLayout_8.addWidget(self.label_6, 1, 0, 1, 1)
        self.doubleSpinBoxDAOFindFWHM = QtWidgets.QDoubleSpinBox(self.tab_4)
        self.doubleSpinBoxDAOFindFWHM.setMinimum(1.0)
        self.doubleSpinBoxDAOFindFWHM.setProperty("value", 3.0)
        self.doubleSpinBoxDAOFindFWHM.setObjectName("doubleSpinBoxDAOFindFWHM")
        self.gridLayout_8.addWidget(self.doubleSpinBoxDAOFindFWHM, 1, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.tab_4)
        self.label_7.setMinimumSize(QtCore.QSize(100, 0))
        self.label_7.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_7.setObjectName("label_7")
        self.gridLayout_8.addWidget(self.label_7, 2, 0, 1, 1)
        self.doubleSpinBoxDAOindThreshold = QtWidgets.QDoubleSpinBox(self.tab_4)
        self.doubleSpinBoxDAOindThreshold.setMinimum(1.0)
        self.doubleSpinBoxDAOindThreshold.setProperty("value", 5.0)
        self.doubleSpinBoxDAOindThreshold.setObjectName("doubleSpinBoxDAOindThreshold")
        self.gridLayout_8.addWidget(self.doubleSpinBoxDAOindThreshold, 2, 1, 1, 1)
        self.gridLayout_9.addLayout(self.gridLayout_8, 0, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_9.addItem(spacerItem4, 1, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_4, "")
        self.gridLayout_10.addWidget(self.tabWidget_2, 6, 0, 1, 2)
        spacerItem5 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_10.addItem(spacerItem5, 7, 1, 1, 1)
        self.comboBoxPhotometryMethods = QtWidgets.QComboBox(self.tab_2)
        self.comboBoxPhotometryMethods.setObjectName("comboBoxPhotometryMethods")
        self.comboBoxPhotometryMethods.addItem("")
        self.comboBoxPhotometryMethods.addItem("")
        self.comboBoxPhotometryMethods.addItem("")
        self.gridLayout_10.addWidget(self.comboBoxPhotometryMethods, 1, 0, 1, 2)
        self.comboBoxExposureInHeader = QtWidgets.QComboBox(self.tab_2)
        self.comboBoxExposureInHeader.setObjectName("comboBoxExposureInHeader")
        self.gridLayout_10.addWidget(self.comboBoxExposureInHeader, 3, 0, 1, 2)
        self.tabWidget.addTab(self.tab_2, "")
        self.gridLayout_4.addWidget(self.tabWidget, 0, 0, 1, 2)

        self.retranslateUi(FormPhotometry)
        self.tabWidget.setCurrentIndex(0)
        self.tabWidget_2.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(FormPhotometry)

    def retranslateUi(self, FormPhotometry):
        _translate = QtCore.QCoreApplication.translate
        FormPhotometry.setWindowTitle(_translate("FormPhotometry", "Photometry"))
        self.pushButtonGO.setText(_translate("FormPhotometry", ":go"))
        self.groupBox_2.setTitle(_translate("FormPhotometry", "Coordinates"))
        item = self.tableWidgetCoordinates.horizontalHeaderItem(0)
        item.setText(_translate("FormPhotometry", "X"))
        item = self.tableWidgetCoordinates.horizontalHeaderItem(1)
        item.setText(_translate("FormPhotometry", "Y"))
        self.groupBox.setTitle(_translate("FormPhotometry", "Image"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("FormPhotometry", "Photometry"))
        self.label.setText(_translate("FormPhotometry", "Kind"))
        self.label_2.setText(_translate("FormPhotometry", "Exposure"))
        self.groupBox_3.setTitle(_translate("FormPhotometry", "Apertures Radius"))
        self.pushButtonAddToRadii.setText(_translate("FormPhotometry", "Add"))
        self.groupBox_4.setTitle(_translate("FormPhotometry", "Header to Extract"))
        self.pushButtonAddToHeaderToExtract.setText(_translate("FormPhotometry", ">>"))
        self.pushButtonRemoveFromHeaderToExtract.setText(_translate("FormPhotometry", "<<"))
        self.label_3.setText(_translate("FormPhotometry", "Sigma"))
        self.label_4.setText(_translate("FormPhotometry", "Minimum Area"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_3), _translate("FormPhotometry", "Source Extract"))
        self.label_5.setText(_translate("FormPhotometry", "Sigma"))
        self.label_6.setText(_translate("FormPhotometry", "FWHM"))
        self.label_7.setText(_translate("FormPhotometry", "Threshold"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_4), _translate("FormPhotometry", "DAOFind"))
        self.comboBoxPhotometryMethods.setItemText(0, _translate("FormPhotometry", "Both"))
        self.comboBoxPhotometryMethods.setItemText(1, _translate("FormPhotometry", "SEP"))
        self.comboBoxPhotometryMethods.setItemText(2, _translate("FormPhotometry", "Photutils"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("FormPhotometry", "Settings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormPhotometry = QtWidgets.QWidget()
    ui = Ui_FormPhotometry()
    ui.setupUi(FormPhotometry)
    FormPhotometry.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ccdproc.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormCCDPROC(object):
    def setupUi(self, FormCCDPROC):
        FormCCDPROC.setObjectName("FormCCDPROC")
        FormCCDPROC.resize(567, 348)
        FormCCDPROC.setMinimumSize(QtCore.QSize(567, 348))
        self.gridLayout_5 = QtWidgets.QGridLayout(FormCCDPROC)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.tabWidget = QtWidgets.QTabWidget(FormCCDPROC)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_2.setMinimumSize(QtCore.QSize(0, 94))
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem = QtWidgets.QSpacerItem(407, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 1, 0, 1, 1)
        self.pushButtonDarkClear = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButtonDarkClear.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonDarkClear.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonDarkClear.setObjectName("pushButtonDarkClear")
        self.gridLayout_2.addWidget(self.pushButtonDarkClear, 1, 2, 1, 1)
        self.pushButtonDarkFile = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButtonDarkFile.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonDarkFile.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonDarkFile.setObjectName("pushButtonDarkFile")
        self.gridLayout_2.addWidget(self.pushButtonDarkFile, 1, 1, 1, 1)
        self.labelDarkFile = QtWidgets.QLabel(self.groupBox_2)
        self.labelDarkFile.setText("")
        self.labelDarkFile.setObjectName("labelDarkFile")
        self.gridLayout_2.addWidget(self.labelDarkFile, 0, 0, 1, 4)
        self.pushButtonDarkShow = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButtonDarkShow.setEnabled(True)
        self.pushButtonDarkShow.setObjectName("pushButtonDarkShow")
        self.gridLayout_2.addWidget(self.pushButtonDarkShow, 1, 3, 1, 1)
        self.gridLayout_6.addWidget(self.groupBox_2, 1, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tab)
        self.groupBox.setMinimumSize(QtCore.QSize(0, 94))
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem1 = QtWidgets.QSpacerItem(407, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 0, 1, 1)
        self.pushButtonZeroShow = QtWidgets.QPushButton(self.groupBox)
        self.pushButtonZeroShow.setEnabled(True)
        self.pushButtonZeroShow.setObjectName("pushButtonZeroShow")
        self.gridLayout.addWidget(self.pushButtonZeroShow, 1, 3, 1, 1)
        self.pushButtonZeroFile = QtWidgets.QPushButton(self.groupBox)
        self.pushButtonZeroFile.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonZeroFile.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonZeroFile.setObjectName("pushButtonZeroFile")
        self.gridLayout.addWidget(self.pushButtonZeroFile, 1, 1, 1, 1)
        self.pushButtonZeroClear = QtWidgets.QPushButton(self.groupBox)
        self.pushButtonZeroClear.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonZeroClear.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonZeroClear.setObjectName("pushButtonZeroClear")
        self.gridLayout.addWidget(self.pushButtonZeroClear, 1, 2, 1, 1)
        self.labelZeroFile = QtWidgets.QLabel(self.groupBox)
        self.labelZeroFile.setText("")
        self.labelZeroFile.setObjectName("labelZeroFile")
        self.gridLayout.addWidget(self.labelZeroFile, 0, 0, 1, 4)
        self.gridLayout_6.addWidget(self.groupBox, 0, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_3.setMinimumSize(QtCore.QSize(0, 94))
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(407, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem2, 1, 0, 1, 1)
        self.pushButtonFlatClear = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonFlatClear.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonFlatClear.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonFlatClear.setObjectName("pushButtonFlatClear")
        self.gridLayout_3.addWidget(self.pushButtonFlatClear, 1, 2, 1, 1)
        self.pushButtonFlatFile = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonFlatFile.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonFlatFile.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonFlatFile.setObjectName("pushButtonFlatFile")
        self.gridLayout_3.addWidget(self.pushButtonFlatFile, 1, 1, 1, 1)
        self.pushButtonFlatShow = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonFlatShow.setEnabled(True)
        self.pushButtonFlatShow.setObjectName("pushButtonFlatShow")
        self.gridLayout_3.addWidget(self.pushButtonFlatShow, 1, 3, 1, 1)
        self.labelFlatFile = QtWidgets.QLabel(self.groupBox_3)
        self.labelFlatFile.setText("")
        self.labelFlatFile.setObjectName("labelFlatFile")
        self.gridLayout_3.addWidget(self.labelFlatFile, 0, 0, 1, 4)
        self.gridLayout_6.addWidget(self.groupBox_3, 2, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(17, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem3, 3, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label = QtWidgets.QLabel(self.tab_2)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 0, 0, 1, 1)
        self.comboBoxExposureHeader = QtWidgets.QComboBox(self.tab_2)
        self.comboBoxExposureHeader.setObjectName("comboBoxExposureHeader")
        self.gridLayout_4.addWidget(self.comboBoxExposureHeader, 1, 0, 1, 1)
        self.checkBoxForce = QtWidgets.QCheckBox(self.tab_2)
        self.checkBoxForce.setObjectName("checkBoxForce")
        self.gridLayout_4.addWidget(self.checkBoxForce, 2, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 160, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem4, 3, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.gridLayout_5.addWidget(self.tabWidget, 0, 0, 1, 2)
        spacerItem5 = QtWidgets.QSpacerItem(460, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_5.addItem(spacerItem5, 1, 0, 1, 1)
        self.pushButtonGO = QtWidgets.QPushButton(FormCCDPROC)
        self.pushButtonGO.setEnabled(True)
        self.pushButtonGO.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setObjectName("pushButtonGO")
        self.gridLayout_5.addWidget(self.pushButtonGO, 1, 1, 1, 1)

        self.retranslateUi(FormCCDPROC)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(FormCCDPROC)

    def retranslateUi(self, FormCCDPROC):
        _translate = QtCore.QCoreApplication.translate
        FormCCDPROC.setWindowTitle(_translate("FormCCDPROC", "CCDPROC"))
        self.groupBox_2.setTitle(_translate("FormCCDPROC", "Master Dark"))
        self.pushButtonDarkClear.setText(_translate("FormCCDPROC", "Clear"))
        self.pushButtonDarkFile.setText(_translate("FormCCDPROC", "Open..."))
        self.pushButtonDarkShow.setText(_translate("FormCCDPROC", "Show"))
        self.groupBox.setTitle(_translate("FormCCDPROC", "Master Zero"))
        self.pushButtonZeroShow.setText(_translate("FormCCDPROC", "Show"))
        self.pushButtonZeroFile.setText(_translate("FormCCDPROC", "Open..."))
        self.pushButtonZeroClear.setText(_translate("FormCCDPROC", "Clear"))
        self.groupBox_3.setTitle(_translate("FormCCDPROC", "Master Flat"))
        self.pushButtonFlatClear.setText(_translate("FormCCDPROC", "Clear"))
        self.pushButtonFlatFile.setText(_translate("FormCCDPROC", "Open..."))
        self.pushButtonFlatShow.setText(_translate("FormCCDPROC", "Show"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("FormCCDPROC", "Calibration"))
        self.label.setText(_translate("FormCCDPROC", "Weight"))
        self.checkBoxForce.setText(_translate("FormCCDPROC", "Force"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("FormCCDPROC", "Settings"))
        self.pushButtonGO.setText(_translate("FormCCDPROC", ":go"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormCCDPROC = QtWidgets.QWidget()
    ui = Ui_FormCCDPROC()
    ui.setupUi(FormCCDPROC)
    FormCCDPROC.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'statics.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormStatics(object):
    def setupUi(self, FormStatics):
        FormStatics.setObjectName("FormStatics")
        FormStatics.resize(818, 221)
        self.gridLayout = QtWidgets.QGridLayout(FormStatics)
        self.gridLayout.setObjectName("gridLayout")
        self.tableWidgetStats = QtWidgets.QTableWidget(FormStatics)
        self.tableWidgetStats.setMinimumSize(QtCore.QSize(800, 0))
        self.tableWidgetStats.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidgetStats.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableWidgetStats.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidgetStats.setObjectName("tableWidgetStats")
        self.tableWidgetStats.setColumnCount(6)
        self.tableWidgetStats.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidgetStats.setHorizontalHeaderItem(5, item)
        self.gridLayout.addWidget(self.tableWidgetStats, 0, 0, 1, 1)

        self.retranslateUi(FormStatics)
        QtCore.QMetaObject.connectSlotsByName(FormStatics)

    def retranslateUi(self, FormStatics):
        _translate = QtCore.QCoreApplication.translate
        FormStatics.setWindowTitle(_translate("FormStatics", "Statics"))
        item = self.tableWidgetStats.horizontalHeaderItem(0)
        item.setText(_translate("FormStatics", "File"))
        item = self.tableWidgetStats.horizontalHeaderItem(1)
        item.setText(_translate("FormStatics", "Npix"))
        item = self.tableWidgetStats.horizontalHeaderItem(2)
        item.setText(_translate("FormStatics", "Mean"))
        item = self.tableWidgetStats.horizontalHeaderItem(3)
        item.setText(_translate("FormStatics", "Stddev"))
        item = self.tableWidgetStats.horizontalHeaderItem(4)
        item.setText(_translate("FormStatics", "Min"))
        item = self.tableWidgetStats.horizontalHeaderItem(5)
        item.setText(_translate("FormStatics", "Max"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormStatics = QtWidgets.QWidget()
    ui = Ui_FormStatics()
    ui.setupUi(FormStatics)
    FormStatics.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'cosmic_cleaner.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormCosmicCleaner(object):
    def setupUi(self, FormCosmicCleaner):
        FormCosmicCleaner.setObjectName("FormCosmicCleaner")
        FormCosmicCleaner.resize(349, 539)
        self.gridLayout_3 = QtWidgets.QGridLayout(FormCosmicCleaner)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.doubleSpinPsffwhm = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinPsffwhm.setMinimum(-9999999999999.0)
        self.doubleSpinPsffwhm.setMaximum(9999999999999.0)
        self.doubleSpinPsffwhm.setProperty("value", 2.5)
        self.doubleSpinPsffwhm.setObjectName("doubleSpinPsffwhm")
        self.gridLayout_2.addWidget(self.doubleSpinPsffwhm, 11, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 3, 0, 1, 1)
        self.spinBoxNiter = QtWidgets.QSpinBox(FormCosmicCleaner)
        self.spinBoxNiter.setMaximum(999999999)
        self.spinBoxNiter.setProperty("value", 4)
        self.spinBoxNiter.setObjectName("spinBoxNiter")
        self.gridLayout_2.addWidget(self.spinBoxNiter, 6, 1, 1, 1)
        self.doubleSpinPsfbeta = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinPsfbeta.setMinimum(-9999999999999.0)
        self.doubleSpinPsfbeta.setMaximum(9999999999999.0)
        self.doubleSpinPsfbeta.setProperty("value", 4.76)
        self.doubleSpinPsfbeta.setObjectName("doubleSpinPsfbeta")
        self.gridLayout_2.addWidget(self.doubleSpinPsfbeta, 13, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 11, 0, 1, 1)
        self.comboBoxFsmode = QtWidgets.QComboBox(FormCosmicCleaner)
        self.comboBoxFsmode.setObjectName("comboBoxFsmode")
        self.comboBoxFsmode.addItem("")
        self.comboBoxFsmode.addItem("")
        self.gridLayout_2.addWidget(self.comboBoxFsmode, 9, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(FormCosmicCleaner)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 6, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 12, 0, 1, 1)
        self.checkBoxGain_apply = QtWidgets.QCheckBox(FormCosmicCleaner)
        self.checkBoxGain_apply.setChecked(True)
        self.checkBoxGain_apply.setObjectName("checkBoxGain_apply")
        self.gridLayout_2.addWidget(self.checkBoxGain_apply, 14, 0, 1, 2)
        self.label_11 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 10, 0, 1, 1)
        self.comboBoxCleantype = QtWidgets.QComboBox(FormCosmicCleaner)
        self.comboBoxCleantype.setObjectName("comboBoxCleantype")
        self.comboBoxCleantype.addItem("")
        self.comboBoxCleantype.addItem("")
        self.comboBoxCleantype.addItem("")
        self.comboBoxCleantype.addItem("")
        self.gridLayout_2.addWidget(self.comboBoxCleantype, 8, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 8, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 13, 0, 1, 1)
        self.doubleSpinObjlim = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinObjlim.setMinimum(-9999999999999.0)
        self.doubleSpinObjlim.setMaximum(9999999999999.0)
        self.doubleSpinObjlim.setProperty("value", 5.0)
        self.doubleSpinObjlim.setObjectName("doubleSpinObjlim")
        self.gridLayout_2.addWidget(self.doubleSpinObjlim, 2, 1, 1, 1)
        self.doubleSpinSatlevel = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinSatlevel.setMinimum(-9999999999999.0)
        self.doubleSpinSatlevel.setMaximum(9999999999999.0)
        self.doubleSpinSatlevel.setProperty("value", 65535.0)
        self.doubleSpinSatlevel.setObjectName("doubleSpinSatlevel")
        self.gridLayout_2.addWidget(self.doubleSpinSatlevel, 5, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 9, 0, 1, 1)
        self.doubleSpinReadnoise = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinReadnoise.setMinimum(-9999999999999.0)
        self.doubleSpinReadnoise.setMaximum(9999999999999.0)
        self.doubleSpinReadnoise.setProperty("value", 6.5)
        self.doubleSpinReadnoise.setObjectName("doubleSpinReadnoise")
        self.gridLayout_2.addWidget(self.doubleSpinReadnoise, 4, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 4, 0, 1, 1)
        self.doubleSpinGain = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinGain.setMinimum(-9999999999999.0)
        self.doubleSpinGain.setMaximum(9999999999999.0)
        self.doubleSpinGain.setProperty("value", 1.0)
        self.doubleSpinGain.setObjectName("doubleSpinGain")
        self.gridLayout_2.addWidget(self.doubleSpinGain, 3, 1, 1, 1)
        self.doubleSpinSigclip = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinSigclip.setMinimum(-99999999999999.0)
        self.doubleSpinSigclip.setMaximum(99999999999999.0)
        self.doubleSpinSigclip.setProperty("value", 4.5)
        self.doubleSpinSigclip.setObjectName("doubleSpinSigclip")
        self.gridLayout_2.addWidget(self.doubleSpinSigclip, 0, 1, 1, 1)
        self.spinBoxPsfsize = QtWidgets.QSpinBox(FormCosmicCleaner)
        self.spinBoxPsfsize.setMaximum(999999999)
        self.spinBoxPsfsize.setProperty("value", 7)
        self.spinBoxPsfsize.setObjectName("spinBoxPsfsize")
        self.gridLayout_2.addWidget(self.spinBoxPsfsize, 12, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 5, 0, 1, 1)
        self.checkBoxSepmed = QtWidgets.QCheckBox(FormCosmicCleaner)
        self.checkBoxSepmed.setChecked(True)
        self.checkBoxSepmed.setObjectName("checkBoxSepmed")
        self.gridLayout_2.addWidget(self.checkBoxSepmed, 7, 0, 1, 2)
        self.label_2 = QtWidgets.QLabel(FormCosmicCleaner)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 1, 0, 1, 1)
        self.doubleSpinSigfrac = QtWidgets.QDoubleSpinBox(FormCosmicCleaner)
        self.doubleSpinSigfrac.setMinimum(-99999999999.0)
        self.doubleSpinSigfrac.setMaximum(99999999999.0)
        self.doubleSpinSigfrac.setProperty("value", 0.3)
        self.doubleSpinSigfrac.setObjectName("doubleSpinSigfrac")
        self.gridLayout_2.addWidget(self.doubleSpinSigfrac, 1, 1, 1, 1)
        self.comboBoxPsfmodel = QtWidgets.QComboBox(FormCosmicCleaner)
        self.comboBoxPsfmodel.setObjectName("comboBoxPsfmodel")
        self.comboBoxPsfmodel.addItem("")
        self.comboBoxPsfmodel.addItem("")
        self.comboBoxPsfmodel.addItem("")
        self.comboBoxPsfmodel.addItem("")
        self.gridLayout_2.addWidget(self.comboBoxPsfmodel, 10, 1, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 2, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 1, 0, 1, 1)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        spacerItem1 = QtWidgets.QSpacerItem(333, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 0, 1, 1)
        self.pushButtonGO = QtWidgets.QPushButton(FormCosmicCleaner)
        self.pushButtonGO.setMinimumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setMaximumSize(QtCore.QSize(80, 25))
        self.pushButtonGO.setObjectName("pushButtonGO")
        self.gridLayout.addWidget(self.pushButtonGO, 0, 1, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout, 2, 0, 1, 1)

        self.retranslateUi(FormCosmicCleaner)
        QtCore.QMetaObject.connectSlotsByName(FormCosmicCleaner)

    def retranslateUi(self, FormCosmicCleaner):
        _translate = QtCore.QCoreApplication.translate
        FormCosmicCleaner.setWindowTitle(_translate("FormCosmicCleaner", "Cosmic Cleaner"))
        self.label_4.setText(_translate("FormCosmicCleaner", "gain"))
        self.label_12.setText(_translate("FormCosmicCleaner", "psffwhm"))
        self.comboBoxFsmode.setItemText(0, _translate("FormCosmicCleaner", "Median"))
        self.comboBoxFsmode.setItemText(1, _translate("FormCosmicCleaner", "Convolve"))
        self.label_3.setText(_translate("FormCosmicCleaner", "objlim"))
        self.label.setText(_translate("FormCosmicCleaner", "sigclip"))
        self.label_8.setText(_translate("FormCosmicCleaner", "niter"))
        self.label_13.setText(_translate("FormCosmicCleaner", "psfsize"))
        self.checkBoxGain_apply.setText(_translate("FormCosmicCleaner", "gain_apply"))
        self.label_11.setText(_translate("FormCosmicCleaner", "psfmodel"))
        self.comboBoxCleantype.setItemText(0, _translate("FormCosmicCleaner", "Median"))
        self.comboBoxCleantype.setItemText(1, _translate("FormCosmicCleaner", "MedMask"))
        self.comboBoxCleantype.setItemText(2, _translate("FormCosmicCleaner", "MeanMask"))
        self.comboBoxCleantype.setItemText(3, _translate("FormCosmicCleaner", "idw"))
        self.label_9.setText(_translate("FormCosmicCleaner", "cleantype"))
        self.label_14.setText(_translate("FormCosmicCleaner", "psfbeta"))
        self.label_10.setText(_translate("FormCosmicCleaner", "fsmode"))
        self.label_5.setText(_translate("FormCosmicCleaner", "readnoise"))
        self.label_6.setText(_translate("FormCosmicCleaner", "satlevel"))
        self.checkBoxSepmed.setText(_translate("FormCosmicCleaner", "sepmed"))
        self.label_2.setText(_translate("FormCosmicCleaner", "sigfrac"))
        self.comboBoxPsfmodel.setItemText(0, _translate("FormCosmicCleaner", "Gauss"))
        self.comboBoxPsfmodel.setItemText(1, _translate("FormCosmicCleaner", "Moffat"))
        self.comboBoxPsfmodel.setItemText(2, _translate("FormCosmicCleaner", "GaussX"))
        self.comboBoxPsfmodel.setItemText(3, _translate("FormCosmicCleaner", "GaussY"))
        self.pushButtonGO.setText(_translate("FormCosmicCleaner", ":go"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    FormCosmicCleaner = QtWidgets.QWidget()
    ui = Ui_FormCosmicCleaner()
    ui.setupUi(FormCosmicCleaner)
    FormCosmicCleaner.show()
    sys.exit(app.exec_())



import tempfile
from pathlib import Path, PurePath
from typing import Optional, Union, List, Tuple, Any

import numpy as np

from .error import NumberOfElementError
from .models import NUMERICS


class Fixer:
    @staticmethod
    def smallest_data_type(data: Any) -> Any:
        """
        Returns the smallest data type for the give array without data loss.

        stolen from: https://stackoverflow.com/questions

        Parameters
        ----------
        data : Any
            the data

        Returns
        -------
        Any
            the smallest data type for the give array
        """
        arr_min = data.min()
        arr_max = data.max()
        for data_type in ["u1", "i1", "u2", "i2", "u4", "i4", "u8", "i8"]:
            iinfo = np.iinfo(np.dtype(data_type))
            if arr_min >= int(iinfo.min) and arr_max <= int(iinfo.max):
                return np.dtype(data_type)

        return np.dtype(np.uint)

    @staticmethod
    def key_value_pair(keys: Union[str, List[str]],
                       values: Union[str, int, float, bool, List[Union[str, int, float, bool]]],
                       comments: Optional[Union[str, List[str]]] = None,
                       ) -> Tuple[List[str], List[Union[str, float, int, bool]], List[str]]:
        """
        Corrects keys, values pair for hedit.

        Parameters
        ----------
        keys : Union[str, List[str]]
            keys to be edited
        values : Union[str, int, float, bool, List[Union[str, int, float, bool]]]
            values to added ot/updated on keys
        comments: Optional[Union[str, List[str]]]
            cpmments to added ot/updated on keys

        Returns
        -------
        Tuple[List[str], List[Union[str, float, int, bool]]]
            tuple of keys and values

        Raises
        ------
        ValueError
            when keys is and values is not a list and vice versa
        """

        if isinstance(keys, str) and not isinstance(values, (str, int, float, bool)):
            raise ValueError("Keys is str so values must be either str, int float, or bool")

        if isinstance(keys, list) and not isinstance(values, list):
            raise ValueError("Keys is list so values must be list")

        keys_to_use = keys if isinstance(keys, list) else [keys]
        values_to_use = values if isinstance(values, list) else [values]

        if comments is None:
            comments_ = [""] * len(values_to_use)
        else:
            if isinstance(comments, list):
                comments_ = comments + [""] * (len(values_to_use) - len(comments))
            else:
                comments_ = [comments] + ([""] * (len(values_to_use) - 1))

        return keys_to_use, values_to_use, comments_

    @staticmethod
    def fitsify(path: str) -> str:
        """
        adds fits if the given path does not end with either `fit` ot `fits`

        Parameters
        ----------
        path : str
            the path to check

        Returns
        -------
        string
            the same path if it ends with either `fit` of `fits` otherwise
            adds `fits` to the end of the path
        """
        if not (path.endswith("fit") or path.endswith("fits")):
            return f"{path}.fits"

        return path

    @staticmethod
    def outputs(output: Optional[str], fits_array) -> Union[List[None], List[str]]:
        """
        Replaces parent directory of the given `fits_array` with the given
        directory `output`. If output is None it will create a temporary one
        in the temp directory

        Parameters
        ----------
        output : str, optional
            directory to replace the parent directory of each file in
            `fits_array`
        fits_array : FitsArray
            `FitsArray` object to change parent directory of each file with
            the given output

        Returns
        -------
        list
            `list` of file paths
        """
        if output is None or not Path(output).is_dir():
            return [None] * len(fits_array)

        to_write = []
        for fits in fits_array:
            f = fits.file
            to_write.append(str(PurePath(output, f.name)))

        return to_write

    @staticmethod
    def output(output: Optional[str] = None, override: bool = False,
               prefix: str = "myraf_", suffix: str = ".fits",
               fitsify: bool = True) -> str:
        """
        Checks for the `output`. If it's `None` creates a temporary file.

        Parameters
        ----------
        output : str, optional
            output file path
        override : bool
            deletes the existing file if `override` is `True`, otherwise it
            raises an error
        prefix : str
            `prefix` value for created temporary file
        suffix : str
            `suffix` value for created temporary file
        fitsify : bool
            makes sure the file ends with either `fit` or `fits`

        Returns
        -------
        Path
            `path` new file

        Raises
        ------
        FileExistsError
            when file already exists and `override` is `False`
        """
        if output is None:
            with tempfile.NamedTemporaryFile(delete=True, prefix=prefix,
                                             suffix=suffix) as f:
                output = f.name

        if fitsify:
            output = Fixer.fitsify(output)

        if Path(output).exists():
            if override:
                Path(output).unlink()
            else:
                raise FileExistsError("File already exist")

        return output

    @classmethod
    def aperture(cls, rs: NUMERICS) -> List[Union[float, int]]:
        """
        Makes sure the given aperture(s) are a list of numbers

        Parameters
        ----------
        rs : NUMERICS
            aperture(s)

        Returns
        -------
        list
            Apertures as `list` of numbers. Even if it is just one aperture
        """
        if isinstance(rs, (float, int)):
            rs = [rs]
        return rs

    @staticmethod
    def header(headers: Optional[Union[str, List[str]]] = None) -> Any:
        """
        Makes sure the given header(s) are a list of headers

        Parameters
        ----------
        headers : Union[str, List[str]], optional
            header(s)

        Returns
        -------
        list
            Headers as `list` of strings. Even if it is just one header
        """

        if headers is None:
            return []

        if isinstance(headers, str):
            headers = [headers]

        return headers

    @staticmethod
    def coordinate(xs: NUMERICS, ys: NUMERICS) -> Tuple[List[Union[float, int]], List[Union[float, int]]]:
        """
        Makes sure the given `x` and `y` coordinate(s) are list of numbers and
        have the same length

        Parameters
        ----------
        xs : NUMERICS
            x coordinate(s)
        ys : NUMERICS
            y coordinate(s)

        Returns
        -------
        Tuple[List[Union[float, int]], List[Union[float, int]]]
            tuple of `xs` and `ys` coordinates

        Raises
        ------
        NumberOfElementError
            when `x` and `y` coordinates does not have the same length
        """
        if isinstance(xs, (float, int)):
            xs = [xs]

        if isinstance(ys, (float, int)):
            ys = [ys]

        if len(xs) != len(ys):
            raise NumberOfElementError("The length of Xs and Ys must be equal")

        return [x for x in xs], [y for y in ys]


class Check:
    @staticmethod
    def operand(operand: str) -> None:
        """
        Checks if the operand is both string and one of `["+", "-", "*", "/", "**", "^"]`

        Parameters
        ----------
        operand : str
            the operand
        Returns
        -------
         None


        Raises
        ------
        ValueError
            when operand is not one of `["+", "-", "*", "/", "**", "^"]`
        """
        if operand not in ["+", "-", "*", "/", "**", "^"]:
            raise ValueError("Operand can only be one of these: +, -, *, /, **, ^")

    @staticmethod
    def method(method: str) -> None:
        """
        Checks if the method is both string and one of `["average", "mean", "median", "sum"]`

        Parameters
        ----------
        method : str
            the method
        Returns
        -------
         None


        Raises
        ------
        ValueError
            when method is not one of `["average", "mean", "median", "sum"]`
        """
        if method not in ["average", "mean", "median", "sum"]:
            raise ValueError("Method can only be one of these: average, mean, median, sum")

    @staticmethod
    def clipping(method: Optional[str] = None) -> None:
        """
        Checks if the clipping is both string and one of `["sigma", "minmax"]`

        Parameters
        ----------
        method : str
            the method
        Returns
        -------
         None


        Raises
        ------
        ValueError
            when method is not one of `["sigma", "minmax"]`
        """
        if method is not None:
            if method not in ["sigma", "minmax"]:
                raise ValueError("Method can only be one of these: sigma, minmax")
